import 'package:flutter/material.dart';

import '../../flutterbudpay.dart';
import '../common/utils.dart';
import '../constant/constant.dart';
import '../constant/submitbutton.dart';
import '../models/checkout_response.dart';
import '../ui/checkout/base_checkout.dart';
import '../ui/checkout/checkout_widget.dart';
import '../ui/error_widget.dart';
import '../ui/sucessful_widget.dart';
import 'transactionstatus.dart';

class Banktransfer extends StatefulWidget {
  final Charge charge;
  final String secretkey, publickey;
  final OnResponse<CheckoutResponse> onResponse;
  const Banktransfer(
      {Key? key,
      required this.secretkey,
      required this.publickey,
      required this.charge,
      required this.onResponse})
      : super(key: key);

  @override
  _BanktransferState createState() => _BanktransferState(charge, onResponse);
}

class _BanktransferState extends BaseCheckoutMethodState<Banktransfer> {
  final Charge _charge;
  _BanktransferState(this._charge, OnResponse<CheckoutResponse> onResponse)
      : super(onResponse, CheckoutMethod.bank);

  String? _paymentError;
  bool _paymentSuccessful = false;

  CheckoutResponse? _response;
  void _onProcessingChange(bool processing) {
    setState(() {
      processing = processing;
    });
  }

  var isloading = false;
  void _onPaymentResponse(CheckoutResponse response) {
    print(response);
    _response = response;
    if (!mounted) return;
    setState(() {
      isloading = false;
    });
    if (response.status == true) {
      _onPaymentSuccess();
    } else {
      _onPaymentError(response.message);
    }
  }

  void _onPaymentSuccess() {
    setState(() {
      _paymentSuccessful = true;
      _paymentError = null;
      _onProcessingChange(false);
    });
  }

  void _onPaymentError(String? value) {
    setState(() {
      _paymentError = value;
      _paymentSuccessful = false;
      _onProcessingChange(false);
    });
  }

  @override
  Widget buildAnimatedChild() {
    // TODO: implement buildAnimatedChild
    return isloading
        ? Transactionstatus(
            charge: _charge,
            secretkey: widget.secretkey,
            verify: true,
            // service: widget.bankService,
            onResponse: _onPaymentResponse,
          )
        : _showProcessingError()
            ? _buildErrorWidget()
            : _paymentSuccessful
                ? _buildSuccessfulWidget()
                : Container(
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        const SizedBox(
                          height: 30,
                        ),
                        Container(
                          decoration: const BoxDecoration(
                              borderRadius:
                                  BorderRadius.all(Radius.circular(17)),
                              color: primarycolour),
                          height: 85,
                          padding: const EdgeInsets.symmetric(horizontal: 20),
                          child: Row(
                            children: [
                              const Text(
                                "Amount",
                                style: TextStyle(
                                    color: Colors.white,
                                    fontSize: 16,
                                    fontWeight: FontWeight.bold),
                              ),
                              const Spacer(),
                              Text(
                                Utils.formatAmount(_charge.amount),
                                style: const TextStyle(
                                    color: Colors.white,
                                    fontSize: 24,
                                    fontWeight: FontWeight.bold),
                              ),
                            ],
                          ),
                        ),
                        const SizedBox(
                          height: 40,
                        ),
                        const Text(
                          "Bank Transfer",
                          style: TextStyle(
                              fontSize: 17.92, fontWeight: FontWeight.bold),
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        const Divider(),
                        const SizedBox(
                          height: 10,
                        ),
                        Row(
                          children: [
                            const Text(
                              "Bank Name",
                              style: TextStyle(
                                  fontSize: 16.13, fontWeight: FontWeight.w500),
                            ),
                            const Spacer(),
                            Text(
                              _charge.account!.bank!.name!,
                              style: const TextStyle(
                                  fontSize: 16.13, fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        const Divider(),
                        const SizedBox(
                          height: 10,
                        ),
                        Row(
                          children: [
                            const Text(
                              "Account Number",
                              style: TextStyle(
                                  fontSize: 16.13, fontWeight: FontWeight.w500),
                            ),
                            const Spacer(),
                            Text(
                              _charge.account!.number!,
                              style: const TextStyle(
                                  fontSize: 16.13, fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        const Divider(),
                        const SizedBox(
                          height: 10,
                        ),
                        Row(
                          children: [
                            const Text(
                              "Account Name",
                              style: TextStyle(
                                  fontSize: 16.13, fontWeight: FontWeight.w500),
                            ),
                            const Spacer(),
                            Text(
                              _charge.account!.name!,
                              style: const TextStyle(
                                  fontSize: 16.13, fontWeight: FontWeight.bold),
                            ),
                          ],
                        ),
                        const SizedBox(
                          height: 10,
                        ),
                        const Divider(),
                        SizedBox(
                          height: 40,
                        ),
                        // const Spacer(),
                        Submitbutton(
                          name: "I have sent the money",
                          press: () {
                            setState(() {
                              isloading = true;
                            });
                          },
                        ),
                        const SizedBox(
                          height: 40,
                        ),
                      ],
                    ),
                  );
  }

  _showProcessingError() {
    return !(_paymentError == null || _paymentError!.isEmpty);
  }

  Widget _buildErrorWidget() {
    return ErrorPage(
      text: _paymentError,
      method: CheckoutMethod.bank,
      isCardPayment: _charge.card!.isValid(),
      vSync: this,
      payWithBank: () {
        setState(() {
          _onPaymentError(null);
          _charge.card = PaymentCard.empty();
          _paymentError = null;
        });
      },
      tryAnotherCard: () {
        setState(() {
          _onPaymentError(null);
          _charge.card = PaymentCard.empty();
        });
      },
      startOverWithCard: () {
        _onPaymentError(null);
      },
    );
  }

  Widget _buildSuccessfulWidget() => SuccessfulWidget(
        amount: _charge.amount,
        onCountdownComplete: () {
          if (_response!.card != null) {
            _response!.card!.nullifyNumber();
          }
          Navigator.of(context).pop(_response);
        },
      );
}
