import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:flutterbudpay/src/wallet/transactionstatus.dart';

import '../../flutterbudpay.dart';
import '../common/utils.dart';
import '../constant/constant.dart';
import '../constant/submitbutton.dart';
import '../models/bank.dart';
import '../models/checkout_response.dart';
import '../request/apis.dart';
import '../ui/checkout/base_checkout.dart';
import '../ui/checkout/checkout_widget.dart';

class Fundwallet extends StatefulWidget {
  final Charge charge;
  final OnResponse<CheckoutResponse> onResponse;
  final String secretKey;
  const Fundwallet(
      {Key? key,
      required this.charge,
      required this.onResponse,
      required this.secretKey})
      : super(key: key);

  @override
  _FundwalletState createState() => _FundwalletState(charge, onResponse);
}

class _FundwalletState extends BaseCheckoutMethodState<Fundwallet> {
  int selected = 1;
  var isLoading = false;
  String ref = "refpay${DateTime.now().microsecondsSinceEpoch}";
  final Charge _charge;
  _FundwalletState(this._charge, OnResponse<CheckoutResponse> onResponse)
      : super(onResponse, CheckoutMethod.bank);

  void initiate() async {
    String res;
    var jsonBody = {
      "email": _charge.email,
      "amount": (_charge.amount ~/ 100).toString(),
      "currency": _charge.currency,
      "reference": _charge.reference
    };
    setState(() {
      isloading = true;
    });
    res = await deposittransfer(jsonBody, widget.secretKey);

    var cmddetails = jsonDecode(res);

    setState(() {
      isloading = false;
    });
    if (cmddetails['status']) {
      setState(() {
        var response = CheckoutResponse(
            message: cmddetails['message'],
            reference: _charge.reference,
            status: false,
            card: _charge.card?..nullifyNumber(),
            method: CheckoutMethod.bank,
            verify: false);
        _charge.account ??= BankAccount(
            Bank(cmddetails['data']['bank_name'], 1),
            cmddetails['data']['account_number'],
            cmddetails['data']['account_name']);
        onResponse(response);
      });
    } else {
      var response = CheckoutResponse(
          message: cmddetails['message'],
          reference: _charge.reference,
          status: false,
          card: _charge.card?..nullifyNumber(),
          method: CheckoutMethod.bank,
          verify: false);
      Navigator.pop(context, response);
    }
  }

  var isloading = false;
  @override
  Widget buildAnimatedChild() {
    return isloading
        ? Transactionstatus(
            charge: _charge,
            secretkey: widget.secretKey,
            verify: false,
            // service: widget.bankService,
            onResponse: _onPaymentResponse,
          )
        : Container(
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                const SizedBox(
                  height: 30,
                ),
                Container(
                  decoration: const BoxDecoration(
                      borderRadius: BorderRadius.all(Radius.circular(17)),
                      color: primarycolour),
                  height: 85,
                  padding: const EdgeInsets.symmetric(horizontal: 20),
                  child: Row(
                    children: [
                      const Text(
                        "Amount",
                        style: TextStyle(
                            color: Colors.white,
                            fontSize: 16,
                            fontWeight: FontWeight.bold),
                      ),
                      const Spacer(),
                      Text(
                        Utils.formatAmount(_charge.amount),
                        style: const TextStyle(
                            color: Colors.white,
                            fontSize: 24,
                            fontWeight: FontWeight.bold),
                      ),
                    ],
                  ),
                ),
                const SizedBox(
                  height: 40,
                ),
                const Text("Transfer Method"),
                const SizedBox(
                  height: 20,
                ),
                GestureDetector(
                  child: Container(
                    height: 85,
                    padding: const EdgeInsets.symmetric(horizontal: 20),
                    decoration: BoxDecoration(
                        // color: Colors.white,
                        border: Border.all(
                            color:
                                selected == 1 ? primarycolour : Colors.white),
                        borderRadius:
                            const BorderRadius.all(Radius.circular(17))),
                    child: Row(
                      children: [
                        Container(
                          height: 20,
                          width: 20,
                          decoration: BoxDecoration(
                              borderRadius:
                                  const BorderRadius.all(Radius.circular(30)),
                              border: Border.all(
                                  color: selected == 1
                                      ? primarycolour
                                      : const Color(0xffD3D3D3),
                                  width: 5)),
                        ),
                        const SizedBox(
                          width: 20,
                        ),
                        const Text("Bank Transfer")
                      ],
                    ),
                  ),
                  onTap: () {
                    setState(() {
                      selected = 1;
                    });
                  },
                ),
                const SizedBox(
                  height: 20,
                ),
                GestureDetector(
                  child: Container(
                    height: 85,
                    padding: const EdgeInsets.symmetric(horizontal: 20),
                    decoration: BoxDecoration(
                        // color: Colors.white,
                        border: Border.all(
                            color:
                                selected == 2 ? primarycolour : Colors.white),
                        borderRadius:
                            const BorderRadius.all(Radius.circular(17))),
                    child: Row(
                      children: [
                        Container(
                          height: 20,
                          width: 20,
                          decoration: BoxDecoration(
                              borderRadius:
                                  const BorderRadius.all(Radius.circular(30)),
                              border: Border.all(
                                  color: selected == 2
                                      ? primarycolour
                                      : const Color(0xffD3D3D3),
                                  width: 5)),
                        ),
                        const SizedBox(
                          width: 20,
                        ),
                        const Text("Debit Card")
                      ],
                    ),
                  ),
                  onTap: () {
                    setState(() {
                      selected = 2;
                    });
                  },
                ),
                const SizedBox(
                  height: 70,
                ),
                Submitbutton(
                    isloading: isLoading,
                    name: "Continue",
                    press: () async {
                      if (selected == 1) {
                        initiate();
                        // onResponse(CheckoutResponse(
                        //     loading: true,
                        //     message: "loading",
                        //     reference: _charge.reference,
                        //     status: false,
                        //     card: _charge.card?..nullifyNumber(),
                        //     method: CheckoutMethod.bank,
                        //     verify: false));
                      } else {
                        onResponse(CheckoutResponse(
                            message: "loading",
                            reference: _charge.reference,
                            status: false,
                            card: _charge.card?..nullifyNumber(),
                            method: CheckoutMethod.card,
                            verify: false));
                      }
                    }),
              ],
            ),
          );
  }

  void _onPaymentResponse(CheckoutResponse response) {
    // print(response);
    // _response = response;
    // if (!mounted) return;
    // setState(() {
    //   isloading = false;
    // });
    // if (response.status == true) {
    //   _onPaymentSuccess();
    // } else {
    //   _onPaymentError(response.message);
    // }
    //
    // if (response.loading && response.method == CheckoutMethod.bank) {
    //   initiate();
    // }
  }
}
