import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:timer_count_down/timer_controller.dart';
import 'package:timer_count_down/timer_count_down.dart';

import '../../../flutterbudpay.dart';
import '../models/checkout_response.dart';
import '../request/apis.dart';
import '../ui/checkout/base_checkout.dart';
import '../ui/checkout/checkout_widget.dart';

class Transactionstatus extends StatefulWidget {
  final Charge charge;
  final bool verify;
  final String secretkey;
  final OnResponse<CheckoutResponse> onResponse;
  const Transactionstatus(
      {Key? key,
      required this.onResponse,
      required this.charge,
      required this.secretkey,
      required this.verify})
      : super(key: key);

  @override
  _TransactionstatusState createState() =>
      _TransactionstatusState(charge, onResponse);
}

class _TransactionstatusState
    extends BaseCheckoutMethodState<Transactionstatus> {
  final Charge _charge;
  _TransactionstatusState(this._charge, OnResponse<CheckoutResponse> onResponse)
      : super(onResponse, CheckoutMethod.bank);
  bool ischange = false;

  final CountdownController _controller =
      new CountdownController(autoStart: true);

  @override
  void initState() {
    super.initState();
    if (widget.verify) {
      initiate();
    }

    startloading();
  }

  void initiate() async {
    String res;

    startloading();
    res = await verifytransaction(_charge.reference, widget.secretkey);
    var cmddetails = jsonDecode(res);
    if (cmddetails['status']) {
      onResponse(CheckoutResponse(
          message: cmddetails['message'],
          reference: cmddetails['data']["reference"],
          status: cmddetails['data']["status"] == "success" ? true : false,
          card: _charge.card?..nullifyNumber(),
          method: CheckoutMethod.bank,
          verify: false));
    } else {
      onResponse(CheckoutResponse(
          message: cmddetails['message'],
          reference: _charge.reference,
          status: false,
          card: _charge.card?..nullifyNumber(),
          method: CheckoutMethod.bank,
          verify: false));
    }
  }

  void startloading() {
    Future.delayed(const Duration(milliseconds: 500), () {
      setState(() {
        ischange = !ischange;
        startloading();
      });
    });
  }

  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
    _controller.pause();
  }

  @override
  Widget buildAnimatedChild() {
    return Container(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Align(
            alignment: Alignment.center,
            child: Image.asset(
              "assets/images/loading${ischange ? "" : "2"}.png",
              width: 126,
              height: 126,
              package: 'flutterbudpay',
            ),
          ),
          Visibility(
            visible: widget.verify,
            child: const Text(
              "Checking transaction status",
              style: TextStyle(fontSize: 18, fontWeight: FontWeight.w500),
            ),
          ),
          const SizedBox(
            height: 5,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              const Text("Please wait ",
                  style: TextStyle(fontSize: 17, fontWeight: FontWeight.w400)),
              Countdown(
                controller: _controller,
                seconds: 300,
                build: (_, double time) => Text(
                  "${Duration(seconds: (time * Duration.secondsPerMinute).round()).toString().split('.')[0].split(':')[0]}:${Duration(seconds: (time * Duration.secondsPerMinute).round()).toString().split('.')[0].split(':')[1]}",
                  style: const TextStyle(
                      fontSize: 17,
                      fontWeight: FontWeight.w400,
                      color: const Color(0xff6C30F4)),
                ),
                interval: const Duration(milliseconds: 1000),
                onFinished: () {
                  ScaffoldMessenger.of(context).showSnackBar(
                    const SnackBar(
                      content: Text('Time expired!'),
                    ),
                  );

                  // setState(() {
                  //   _timerShow = false;
                  // });
                },
              ),
            ],
          )
        ],
      ),
    );
  }
}
